static const char _rcsid_[]="$Id: error.c,v 1.11 2001/12/28 08:13:05 gerhard Exp $";
/* Copyright Gerhard Rieger 2001 */
/* Published under the GNU General Public License V.2, see file COPYING */

/* the logging subsystem */

#include "config.h"

#include <stdarg.h>
#include <stdlib.h>
#include <errno.h>
#if HAVE_SYSLOG_H
#include <syslog.h>
#endif
#include <time.h>	/* time_t, strftime() */
#include <stdio.h>
#include <string.h>
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#include "mytypes.h"
#include "compat.h"

#include "error.h"

/* translate MSG level to SYSLOG level */
int syslevel[] = {
   LOG_DEBUG,
   LOG_INFO,
   LOG_NOTICE,
   LOG_WARNING,
   LOG_ERR,
   LOG_CRIT };

struct diag_opts {
   const char *progname;
   int msglevel;
   int exitlevel;
   int logstderr;
   int syslog;
   FILE *logfile;
   int logfacility;
} ;


struct diag_opts diagopts =
  { NULL, E_ERROR, E_ERROR, 1, 0, NULL, LOG_DAEMON } ;

static void _msg(int level, const char *buff, const char *syslp);


void diag_set(char what, const char *arg) {
   switch (what) {
   case 'y': diagopts.syslog = true;
      openlog("socat", LOG_PID, diagopts.logfacility);
      diagopts.logstderr = false; break;
   case 'f': diagopts.logfile = fopen(arg, "a");
      diagopts.logstderr = false; break;
   case 's': diagopts.logstderr = true; break;	/* logging to stderr is default */
   case 'p': diagopts.progname = arg; break;
   case 'd': --diagopts.msglevel; break;
   case 'e': diagopts.exitlevel = strtoul(arg, NULL, 10); break;
   default: msg(E_ERROR, "unknown log device specification %c%s", what, arg);
   }
}

/* Linux and AIX syslog format:
Oct  4 17:10:37 hostname socat[52798]: D signal(13, 1)
*/
void msg(int level, const char *format, ...) {
   time_t now;
#define BUFLEN 256
   char buff[BUFLEN], *bufp, *syslp;
   va_list ap;

   va_start(ap, format);
   if (level < diagopts.msglevel)  return;
   now = time(NULL);  if (now == (time_t)-1) {
      /* invoking msg() might create endless recursion; by hand instead */
      sprintf(buff, "cannot read time:   %s["F_pid"] E %s",
	      diagopts.progname, getpid(), strerror(errno));
      _msg(LOG_ERR, buff, strstr(buff, " E "+1));
      strcpy(buff, "unknown time        ");
   } else {
#if HAVE_STRFTIME
      strftime(buff, 21, "%Y/%m/%d %H:%M:%S ", localtime(&now));
#else
      strcpy(buff, 21, ctime(&now));
#endif
   }
   sprintf(buff+20, "%s["F_pid"] ", diagopts.progname, getpid());
   syslp = strchr(buff+23, '\0');
   sprintf(syslp, "%c ", "DINWEF"[level]);
   bufp = strchr(syslp, '\0');
   vsnprintf(bufp, BUFLEN-35-1, format, ap);
   strcat(bufp, "\n");
   _msg(level, buff, syslp);
   if (level >= diagopts.exitlevel) {
      va_end(ap);
      exit(1);
   }
   va_end(ap);
}


static void _msg(int level, const char *buff, const char *syslp) {
   if (diagopts.logstderr) {
      fputs(buff, stderr); fflush(stderr);
   }
   if (diagopts.syslog) {
      syslog(syslevel[level], syslp);
   }
   if (diagopts.logfile) {
      fputs(buff, diagopts.logfile); fflush(diagopts.logfile);
   }
}
