static const char _rcsid_[]="$Id: xioshutdown.c,v 1.7 2001/07/15 21:09:18 gerhard Exp $";
/* Copyright Gerhard Rieger 2001 */
/* Published under the GNU General Public License V.2, see file COPYING */

/* this is the source of the extended shutdown function */


#include "xiosysincludes.h"
#include "xioopen.h"


int xioshutdown(xiofile_t *sock, int how) {
   int result = 0;

   if (sock->tag == XIO_TAG_INVALID) {
      Error("xioread(): invalid file descriptor");
      errno = EINVAL;
      return -1;
   }

   if (sock->tag == XIO_TAG_DUAL) {
      if ((how+1)&1) {
	 result = xioshutdown(sock->dual.stream[0], 0);
      }
      if ((how+1)&2) {
	 result |= xioshutdown(sock->dual.stream[1], 1);
      }

   } else if (sock->stream.dtype == DATA_PIPE) {
      if ((how+1)&1) {
	 if (Close(sock->stream.fd) < 0) {
	    Warn2("close(%d): %s",
		  sock->stream.fd, strerror(errno));
	 }
      }
      if ((how+1)&2) {
	 if (Close(sock->stream.para.bipipe.fdout) < 0) {
	    Warn2("close(%d): %s",
		  sock->stream.para.bipipe.fdout, strerror(errno));
	 } 
      }
      
   } else if (sock->stream.dtype == DATA_2PIPE) {
      if ((how+1)&1) {
	 if (Close(sock->stream.fd) < 0) {
	    Warn2("close(%d): %s",
		  sock->stream.fd, strerror(errno));
	 }
      }
      if ((how+1)&2) {
	 if (Close(sock->stream.para.exec.fdout) < 0) {
	    Warn2("close(%d): %s",
		  sock->stream.para.exec.fdout, strerror(errno));
	 } 
      }
      
   } else if (sock->stream.dtype == DATA_RECV) {
#if WITH_SOCKET
      if (sock->stream.howtoend == END_SHUTDOWN) {
	 if ((result = Shutdown(sock->stream.fd, how)) < 0) {
	    Warn3("shutdown(%d, %d): %s",
		  sock->stream.fd, how, strerror(errno));
	 }
      } else
#endif /* WITH_SOCKET */
      {
	 if (how >= 1) {
	    if (Close(sock->stream.fd) < 0) {
	       Warn2("close(%d): %s",
		     sock->stream.fd, strerror(errno));
	    }
	    sock->stream.fd = -1;
	 }
      }
   }
   return result;
}
